package ci;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import com.jgoodies.looks.plastic.PlasticXPLookAndFeel;

/*
 * This class simplifies the creation of self-drawing components
 * like charts. The basic implemented behavior is the ability
 * to pan and zoom via mouse.
 */
public abstract class DrawingComponent extends JComponent
{
    /** the transformation determining the current view */
    protected AffineTransform         m_atZoomAndMove     = AffineTransform.getScaleInstance(1.2, 1.2);


    /*
     * The next members are only valid between mouse pressed and release!
     */
    /** point where the mouse was pressed */
    protected Point2D.Double          m_ptInitial         = null;
    /** that last mouse location during dragging */
    protected Point2D.Double          m_ptLast            = null;

    private double m_dScaleX = 60, m_dScaleY  = 80;
    
    
    /**
     * C'tor
     */
    public DrawingComponent()
    {
        /*
         * Install Listeners
         * -----------------
         */
        this.addMouseWheelListener(new MouseWheelListener(){

            public void mouseWheelMoved(MouseWheelEvent ev)
            {
                view_mouseWheelMoved(ev);
            }
        });
        this.addMouseMotionListener(new MouseMotionListener(){

            public void mouseDragged(MouseEvent ev)
            {
                view_mouseDragged(ev);
            }

            public void mouseMoved(MouseEvent ev)
            {
                view_mouseMoved(ev);
            }
            
        });
        this.addMouseListener(new MouseListener(){

            public void mouseClicked(MouseEvent ev)
            {
                view_mouseClicked(ev);
            }

            public void mouseEntered(MouseEvent ev)
            {
                view_mouseEntered(ev);
            }

            public void mouseExited(MouseEvent ev)
            {
                view_mouseExited(ev);
            }

            public void mousePressed(MouseEvent ev)
            {
                view_mousePressed(ev);
            }

            public void mouseReleased(MouseEvent ev)
            {
                view_mouseReleased(ev);
            }
            
        });
        
        
        setOpaque(true);
        m_atZoomAndMove.translate(150, 260);
    }
 
    
    /**
     * Converts screen coordinates (pt) into the view's
     * coordinate system by inverting <code>m_atZoomAndMove</code>.
     * 
     * @param   ptScreen screen coordinates
     * @return point in graph's coordinates
     */
    public Point2D.Double getUserCoordsFromScreenCoords(Point ptScreen)
    {
        if (ptScreen == null)
            return new Point2D.Double(0, 0);
        
        try
        {
            Point2D pt2D = m_atZoomAndMove.inverseTransform(new Point2D.Double(ptScreen.getX(), ptScreen.getY()), null);

            if (pt2D instanceof Point2D.Double)
                return (Point2D.Double)pt2D;
            else
                return new Point2D.Double(pt2D.getX(), pt2D.getY());
            
        } catch (NoninvertibleTransformException e)
        {
            e.printStackTrace();
            return null;
        }
        
    }
    
    public Double getDataCoordsFromScreenCoords(Point ptScreen)
    {
        Point2D.Double ptUser = getUserCoordsFromScreenCoords(ptScreen);
        if (ptUser == null)
            return null;
        
        return new Point2D.Double( ptUser.getX()/m_dScaleX, ptUser.getY()/m_dScaleY);
    }
    
    /**
     * Computes a vector from ptB to ptA.
     * 
     * @param   ptA point A
     * @param   ptB point B
     * @return vector B-A
     */
    public static Point2D.Double subtract(Point2D ptA, Point2D ptB)
    {
        return new Point2D.Double( ptA.getX()-ptB.getX() , ptA.getY()-ptB.getY() );
    }

    public static Point2D.Double add(Point2D ptA, Point2D ptB)
    {
        return new Point2D.Double( ptA.getX()+ptB.getX() , ptA.getY()+ptB.getY() );
    }
    
    public static double length(Point2D v)
    {
        return Math.sqrt(v.getX()*v.getX() + v.getY()*v.getY());
    }

    

    protected void view_mouseReleased(MouseEvent ev)
    {
        
    }


    protected void view_mousePressed(MouseEvent ev)
    {
        /* Remember where the mouse was pressed */
        m_ptInitial = getUserCoordsFromScreenCoords( ev.getPoint() );
        m_ptLast    = m_ptInitial;
    }
    


    protected void view_mouseExited(MouseEvent ev)
    {
        this.setCursor(Cursor.getDefaultCursor());
    }


    protected void view_mouseEntered(MouseEvent ev)
    {
        this.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    }


    protected void view_mouseClicked(MouseEvent ev)
    {
        
    }


    protected void view_mouseMoved(MouseEvent ev)
    {
        
    }


    protected void view_mouseDragged(MouseEvent ev)
    {
        Point2D.Double ptCurrent        = getUserCoordsFromScreenCoords( ev.getPoint() );
        Point2D.Double ptDiffInitial    = subtract(m_ptInitial, ptCurrent);
        
        if (SwingUtilities.isRightMouseButton(ev))
        {
            m_atZoomAndMove.translate(-ptDiffInitial.getX(), -ptDiffInitial.getY());
        
            repaint();
        }
        
    }


    protected void view_mouseWheelMoved(MouseWheelEvent ev)
    {
        double dFactor = Math.signum(ev.getWheelRotation())<0?1.07:0.93;
        Point2D.Double ptCurrent = getUserCoordsFromScreenCoords( ev.getPoint() );
    
        /* The two translations are used to let the mouse pointer
         * represent the location where to zoom in or out */
        m_atZoomAndMove.translate(ptCurrent.getX(), ptCurrent.getY());
        m_atZoomAndMove.scale(dFactor, dFactor);
        m_atZoomAndMove.translate(-ptCurrent.getX(), -ptCurrent.getY());
        
        repaint();
    }
    
    @Override
    protected void paintComponent(Graphics g)
    {
        if (isOpaque())
        {
            g.setColor(UIManager.getColor("control"));
            g.fillRect(0, 0, getWidth(), getHeight());
        }

        
        Graphics2D g2 = (Graphics2D)g;
        
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        /* Move the coordinate system according to
         * the current view and zoom state. */
        g2.transform(m_atZoomAndMove);
            
    }
    
    /**
     * Draws a debugging coordinate cross.
     * 
     * @param g the context to draw to
     */
    private void debugRulers(Graphics g)
    {
        if (g == null)
            return;
        
        Graphics2D g2 = (Graphics2D)g.create();
        
        try
        {
            GeneralPath shpArrowRight = new GeneralPath();
            shpArrowRight.moveTo(500f, 0);
            shpArrowRight.lineTo(500f, 5);
            shpArrowRight.lineTo(515f, 0);
            shpArrowRight.lineTo(500f, -5);
            shpArrowRight.lineTo(500f, 0);

            GeneralPath shpArrowUp = new GeneralPath();
            shpArrowUp.moveTo(0f,  500);
            shpArrowUp.lineTo(5f,  500);
            shpArrowUp.lineTo(0f,  515);
            shpArrowUp.lineTo(-5,  500);
            shpArrowUp.lineTo(0f,  500);
            
            
            Line2D.Float lineXAxis = new Line2D.Float(0, 0, 500f, 0);
            Line2D.Float lineYAxis = new Line2D.Float(0, 0f, 0, 500f);
            g2.setColor(new Color(0, 0, 0));
            g2.draw(lineXAxis);
            g2.draw(lineYAxis);
            g2.fill(shpArrowRight);
            g2.fill(shpArrowUp);

            

        } finally
        {
            g2.dispose();
        }
    }
    
    
    
//    public AffineTransform getTransform()
//    {
//        return m_atZoomAndMove;
//    }
    


    public void zoom100()
    {
        m_atZoomAndMove     = AffineTransform.getScaleInstance(1, 1);
        
        repaint();
    }

    public double getScaleX()
    {
        return m_dScaleX;
    }


    public double getScaleY()
    {
        return m_dScaleY;
    }
    
    
    
}
