package ci;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Random;

import org.apache.commons.math.MathException;
import org.apache.commons.math.distribution.ChiSquaredDistribution;
import org.apache.commons.math.distribution.DistributionFactory;
import org.apache.commons.math.distribution.ExponentialDistribution;

public class ExponentialDensity implements IDensity
{
	private ExponentialDistribution m_dist = null;
	private Random m_rnd = new Random(System.currentTimeMillis());
	protected double m_theta = 1;
	protected Color m_color = Color.BLUE;
	protected double m_ci_borders[][];
	
	public ExponentialDensity(double theta)
	{
		setParameter(theta);
	}
	
	public double f(double x)
	{
		return (1/m_theta) * Math.exp(-x/m_theta);
	}

	public double getParameter()
	{
		return m_theta;
	}

	public void setParameter(double theta)
	{
		m_dist = DistributionFactory.newInstance().createExponentialDistribution(theta);
		m_theta = theta;
		m_ci_borders = null;
	}
	public double getXmax()
	{
		return getParameter() * 7;
	}

	public double getXmin()
	{
		return 0.0;
	}

	public double getYmax()
	{
		return 1./m_theta;
	}

	public double getYmin()
	{
		return 0.0;
	}

	public void draw(Graphics g, IScale s)
	{
		Graphics2D g2 = (Graphics2D)g.create();
        try
        {
        	
        	/*
        	 * Draw CIs if present
        	 */
        	if (m_ci_borders != null)
        	{
    			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.05f));
        		for (int i=0; i<m_ci_borders.length; i++)
        		{
        			double top = 0;
        			if (m_ci_borders[i][0]<m_theta && m_theta < m_ci_borders[i][1])
        			{
            			g2.setPaint(new Color(0f,0.8f,0f));
        				
        			} else
        			{
            			g2.setPaint(Color.RED);
            			top = -0.01;
        			}

        			Rectangle2D.Double interval = new Rectangle2D.Double(
            				m_ci_borders[i][0]*s.getXScale(), 
            				top*s.getYScale(), 
            				(m_ci_borders[i][1]-m_ci_borders[i][0])*s.getXScale(), 
            				0.01*s.getYScaleAbs()
            			);
        			
        			g2.fill(interval);
        		}
        	}
        	
        	g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC));
        	GeneralPath graph = new GeneralPath();
        	// draw the density
        	double xstep = 0.1;
        	double x = this.getXmin();
        	double y = Double.NaN;
        	boolean first = true;
        	while (x < this.getXmax())
        	{
        		try
        		{
        			y = this.f(x);
        			if (first)
        			{
        				graph.moveTo((float)(x*s.getXScale()), (float)(y*s.getYScale()));
        				first = false;
        			} else
        				graph.lineTo((float)(x*s.getXScale()), (float)(y*s.getYScale()));
        			
        		} catch (Exception ex)
        		{
        		}

    			x += xstep;
        	}
        	
        	g2.setColor(m_color);
        	g2.draw(graph);
        	
        	drawXTick(g2, m_theta, s);
        
        	g2.drawString(String.valueOf(m_theta), (float)(m_theta*s.getXScale())-8f, (float)(-0.018*s.getYScale()));
        	
        } finally
        {
            g2.dispose();
        }
		
		
	}


	private void drawXTick(Graphics2D g2, double x, IScale s)
	{
    	g2.setColor(Color.BLACK);
    	g2.draw(new Line2D.Double(m_theta*s.getXScale(), -0.005*s.getYScale(), m_theta*s.getXScale(), 0.005*s.getYScale()));
	}

	public int sampleAndCalcCI(int size, int count, double alpha)
	{
		int good_ci = 0;
		double[] sample = null;
		m_ci_borders = new double[count][];
		for (int i=0; i<count; i++)
		{
			/*
			 * Draw a sample
			 */
			sample = new double[size];
			for (int j=0; j<size; j++)
			{
				try
				{
					sample[j] = m_dist.inverseCumulativeProbability(m_rnd.nextDouble());
					
				} catch (MathException e)
				{
					e.printStackTrace();
				}
			}
			
			/*
			 * Calculate CI for sample
			 */
	    	ChiSquaredDistribution chi2dist = DistributionFactory.newInstance().createChiSquareDistribution(2*size);
	    	
	    	double mean = 0;
	    	for (int j=0; j<size; j++)
	    		mean += sample[j];
	    	mean /= size;

	    	m_ci_borders[i] = new double[2];
	    	try
	    	{
	    		m_ci_borders[i][0] = mean * 2*size / chi2dist.inverseCumulativeProbability(1-alpha/2.);
	    		m_ci_borders[i][1] = mean * 2*size / chi2dist.inverseCumulativeProbability(alpha/2.);
	    		
	    		if (m_ci_borders[i][0] < m_theta && m_theta < m_ci_borders[i][1])
	    			good_ci++;
	    		
	    	} catch (MathException mex)
	    	{
	    		mex.printStackTrace();
	    	}
			
		}
		
		return good_ci;
	}
}
